/*
 * DDESBFView.java
 */

package ddesbf;

import des.DoubleDes;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

/**
 * The application's main frame.
 */
public class DDESBFView extends FrameView {

    public DDESBFView(SingleFrameApplication app) {
        super(app);

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = DDESBFApp.getApplication().getMainFrame();
            aboutBox = new DDESBFAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        DDESBFApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanel1 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        txtPlainText = new javax.swing.JTextArea();
        jPanel4 = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        txtCypherText = new javax.swing.JTextArea();
        btnCrypt = new javax.swing.JButton();
        btnDecrypt = new javax.swing.JButton();
        btnSetPair1 = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        tfInputKey = new javax.swing.JTextField();
        btnSetPair2 = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        tfKeyFound = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        btnAttack = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        jRadioButton3 = new javax.swing.JRadioButton();
        jRadioButton4 = new javax.swing.JRadioButton();
        jRadioButton5 = new javax.swing.JRadioButton();
        jRadioButton6 = new javax.swing.JRadioButton();
        jLabel8 = new javax.swing.JLabel();
        cbAlphabetType = new javax.swing.JComboBox();
        jLabel9 = new javax.swing.JLabel();
        tfEstimatedTime = new javax.swing.JTextField();
        btnTimeTest = new javax.swing.JButton();
        jLabel10 = new javax.swing.JLabel();
        tfPath = new javax.swing.JTextField();
        btnPath = new javax.swing.JButton();
        jLabel11 = new javax.swing.JLabel();
        tfKeyOriginal = new javax.swing.JTextField();
        jPanel5 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jScrollPane3 = new javax.swing.JScrollPane();
        tfPair1PlainText = new javax.swing.JTextArea();
        jScrollPane4 = new javax.swing.JScrollPane();
        tfPair1CypherText = new javax.swing.JTextArea();
        jPanel6 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jScrollPane5 = new javax.swing.JScrollPane();
        tfPair2CypherText = new javax.swing.JTextArea();
        jScrollPane6 = new javax.swing.JScrollPane();
        tfPair2PlainText = new javax.swing.JTextArea();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        miOpenPlainTextFile = new javax.swing.JMenuItem();
        miOpenCypherTextFile = new javax.swing.JMenuItem();
        javax.swing.JMenuItem miExit = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        fcOpenFile = new javax.swing.JFileChooser();
        bgTableSize = new javax.swing.ButtonGroup();
        bgCouple = new javax.swing.ButtonGroup();

        mainPanel.setName("mainPanel"); // NOI18N

        jTabbedPane1.setName("jTabbedPane1"); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(ddesbf.DDESBFApp.class).getContext().getResourceMap(DDESBFView.class);
        jPanel3.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel3.border.title"))); // NOI18N
        jPanel3.setName("jPanel3"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        txtPlainText.setColumns(20);
        txtPlainText.setRows(5);
        txtPlainText.setText(resourceMap.getString("txtPlainText.text")); // NOI18N
        txtPlainText.setName("txtPlainText"); // NOI18N
        jScrollPane1.setViewportView(txtPlainText);

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 536, Short.MAX_VALUE)
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 169, Short.MAX_VALUE)
        );

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel4.border.title"))); // NOI18N
        jPanel4.setName("jPanel4"); // NOI18N

        jScrollPane2.setName("jScrollPane2"); // NOI18N

        txtCypherText.setColumns(20);
        txtCypherText.setRows(5);
        txtCypherText.setName("txtCypherText"); // NOI18N
        jScrollPane2.setViewportView(txtCypherText);

        org.jdesktop.layout.GroupLayout jPanel4Layout = new org.jdesktop.layout.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 536, Short.MAX_VALUE)
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 186, Short.MAX_VALUE)
        );

        btnCrypt.setText(resourceMap.getString("btnCrypt.text")); // NOI18N
        btnCrypt.setName("btnCrypt"); // NOI18N
        btnCrypt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCryptActionPerformed(evt);
            }
        });

        btnDecrypt.setText(resourceMap.getString("btnDecrypt.text")); // NOI18N
        btnDecrypt.setName("btnDecrypt"); // NOI18N
        btnDecrypt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDecryptActionPerformed(evt);
            }
        });

        btnSetPair1.setText(resourceMap.getString("btnSetPair1.text")); // NOI18N
        btnSetPair1.setName("btnSetPair1"); // NOI18N
        btnSetPair1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSetPair1ActionPerformed(evt);
            }
        });

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        tfInputKey.setText(resourceMap.getString("tfInputKey.text")); // NOI18N
        tfInputKey.setName("tfInputKey"); // NOI18N

        btnSetPair2.setText(resourceMap.getString("btnSetPair2.text")); // NOI18N
        btnSetPair2.setName("btnSetPair2"); // NOI18N
        btnSetPair2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSetPair2ActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jLabel1)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(tfInputKey, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 155, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 41, Short.MAX_VALUE)
                .add(btnCrypt)
                .add(18, 18, 18)
                .add(btnDecrypt)
                .addContainerGap())
            .add(jPanel4, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap(322, Short.MAX_VALUE)
                .add(btnSetPair1)
                .add(11, 11, 11)
                .add(btnSetPair2)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnDecrypt)
                    .add(btnCrypt)
                    .add(jLabel1)
                    .add(tfInputKey, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(1, 1, 1)
                .add(jPanel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btnSetPair1)
                    .add(btnSetPair2))
                .add(60, 60, 60))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel1.TabConstraints.tabTitle"), jPanel1); // NOI18N

        jPanel2.setName("jPanel2"); // NOI18N

        jPanel8.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel8.border.title"))); // NOI18N
        jPanel8.setName("jPanel8"); // NOI18N

        tfKeyFound.setEditable(false);
        tfKeyFound.setText(resourceMap.getString("tfKeyFound.text")); // NOI18N
        tfKeyFound.setName("tfKeyFound"); // NOI18N

        jLabel6.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        jLabel6.setName("jLabel6"); // NOI18N

        btnAttack.setText(resourceMap.getString("btnAttack.text")); // NOI18N
        btnAttack.setName("btnAttack"); // NOI18N
        btnAttack.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnAttackActionPerformed(evt);
            }
        });

        jLabel7.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        jLabel7.setName("jLabel7"); // NOI18N

        bgTableSize.add(jRadioButton3);
        jRadioButton3.setText(resourceMap.getString("jRadioButton3.text")); // NOI18N
        jRadioButton3.setActionCommand(resourceMap.getString("jRadioButton3.actionCommand")); // NOI18N
        jRadioButton3.setName("jRadioButton3"); // NOI18N

        bgTableSize.add(jRadioButton4);
        jRadioButton4.setSelected(true);
        jRadioButton4.setText(resourceMap.getString("jRadioButton4.text")); // NOI18N
        jRadioButton4.setActionCommand(resourceMap.getString("jRadioButton4.actionCommand")); // NOI18N
        jRadioButton4.setName("jRadioButton4"); // NOI18N

        bgTableSize.add(jRadioButton5);
        jRadioButton5.setText(resourceMap.getString("jRadioButton5.text")); // NOI18N
        jRadioButton5.setActionCommand(resourceMap.getString("jRadioButton5.actionCommand")); // NOI18N
        jRadioButton5.setName("jRadioButton5"); // NOI18N

        bgTableSize.add(jRadioButton6);
        jRadioButton6.setText(resourceMap.getString("jRadioButton6.text")); // NOI18N
        jRadioButton6.setActionCommand(resourceMap.getString("jRadioButton6.actionCommand")); // NOI18N
        jRadioButton6.setName("jRadioButton6"); // NOI18N

        jLabel8.setText(resourceMap.getString("jLabel8.text")); // NOI18N
        jLabel8.setName("jLabel8"); // NOI18N

        cbAlphabetType.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "numeric (0,1,2,..)", "lower case (a,b,c,..)", "upper case(A,B,C,..)", "numeric & lower case (0,1,2,..,a,b,c,..)", "numeric & upper case (0,1,2,..,A,B,C,..)", "all" }));
        cbAlphabetType.setSelectedIndex(1);
        cbAlphabetType.setName("cbAlphabetType"); // NOI18N

        jLabel9.setText(resourceMap.getString("jLabel9.text")); // NOI18N
        jLabel9.setName("jLabel9"); // NOI18N

        tfEstimatedTime.setEditable(false);
        tfEstimatedTime.setName("tfEstimatedTime"); // NOI18N

        btnTimeTest.setText(resourceMap.getString("btnTimeTest.text")); // NOI18N
        btnTimeTest.setName("btnTimeTest"); // NOI18N
        btnTimeTest.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnTimeTestActionPerformed(evt);
            }
        });

        jLabel10.setText(resourceMap.getString("jLabel10.text")); // NOI18N
        jLabel10.setName("jLabel10"); // NOI18N

        tfPath.setText(resourceMap.getString("tfPath.text")); // NOI18N
        tfPath.setName("tfPath"); // NOI18N

        btnPath.setText(resourceMap.getString("btnPath.text")); // NOI18N
        btnPath.setName("btnPath"); // NOI18N
        btnPath.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnPathActionPerformed(evt);
            }
        });

        jLabel11.setText(resourceMap.getString("jLabel11.text")); // NOI18N
        jLabel11.setName("jLabel11"); // NOI18N

        tfKeyOriginal.setEditable(false);
        tfKeyOriginal.setName("tfKeyOriginal"); // NOI18N

        org.jdesktop.layout.GroupLayout jPanel8Layout = new org.jdesktop.layout.GroupLayout(jPanel8);
        jPanel8.setLayout(jPanel8Layout);
        jPanel8Layout.setHorizontalGroup(
            jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel8Layout.createSequentialGroup()
                            .add(jLabel7)
                            .add(37, 37, 37)
                            .add(jRadioButton3)
                            .add(18, 18, 18)
                            .add(jRadioButton4)
                            .add(18, 18, 18)
                            .add(jRadioButton5)
                            .add(18, 18, 18)
                            .add(jRadioButton6))
                        .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel8Layout.createSequentialGroup()
                            .add(jLabel8)
                            .add(21, 21, 21)
                            .add(cbAlphabetType, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                    .add(jPanel8Layout.createSequentialGroup()
                        .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(btnTimeTest)
                            .add(btnAttack))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel8Layout.createSequentialGroup()
                                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(jPanel8Layout.createSequentialGroup()
                                        .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                            .add(jLabel9)
                                            .add(jLabel6))
                                        .add(21, 21, 21))
                                    .add(jPanel8Layout.createSequentialGroup()
                                        .add(jLabel11)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)))
                                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(tfKeyOriginal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 236, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(tfKeyFound, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 236, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(tfEstimatedTime, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 236, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                            .add(jPanel8Layout.createSequentialGroup()
                                .add(tfPath, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 252, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(btnPath))))
                    .add(jLabel10))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel8Layout.setVerticalGroup(
            jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel8Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel7)
                    .add(jRadioButton3)
                    .add(jRadioButton4)
                    .add(jRadioButton5)
                    .add(jRadioButton6))
                .add(18, 18, 18)
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel8)
                    .add(cbAlphabetType, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(18, 18, 18)
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel10)
                    .add(tfPath, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnPath))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel9)
                    .add(tfEstimatedTime, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnTimeTest))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel6)
                    .add(tfKeyFound, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnAttack))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel8Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(tfKeyOriginal, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel11))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel5.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel5.border.title"))); // NOI18N
        jPanel5.setName("jPanel5"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jScrollPane3.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane3.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPane3.setName("jScrollPane3"); // NOI18N

        tfPair1PlainText.setColumns(20);
        tfPair1PlainText.setRows(5);
        tfPair1PlainText.setName("tfPair1PlainText"); // NOI18N
        jScrollPane3.setViewportView(tfPair1PlainText);

        jScrollPane4.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane4.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPane4.setName("jScrollPane4"); // NOI18N

        tfPair1CypherText.setColumns(20);
        tfPair1CypherText.setRows(5);
        tfPair1CypherText.setName("tfPair1CypherText"); // NOI18N
        jScrollPane4.setViewportView(tfPair1CypherText);

        org.jdesktop.layout.GroupLayout jPanel5Layout = new org.jdesktop.layout.GroupLayout(jPanel5);
        jPanel5.setLayout(jPanel5Layout);
        jPanel5Layout.setHorizontalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(jLabel3))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(jScrollPane4)
                    .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 321, Short.MAX_VALUE))
                .addContainerGap(70, Short.MAX_VALUE))
        );
        jPanel5Layout.setVerticalGroup(
            jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5Layout.createSequentialGroup()
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel5Layout.createSequentialGroup()
                        .add(31, 31, 31)
                        .add(jLabel2))
                    .add(jPanel5Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jScrollPane3, 0, 0, Short.MAX_VALUE)))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel5Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jLabel3)
                    .add(jScrollPane4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(48, 48, 48))
        );

        jPanel6.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel6.border.title"))); // NOI18N
        jPanel6.setName("jPanel6"); // NOI18N

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        jScrollPane5.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane5.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPane5.setName("jScrollPane5"); // NOI18N

        tfPair2CypherText.setColumns(20);
        tfPair2CypherText.setRows(5);
        tfPair2CypherText.setName("tfPair2CypherText"); // NOI18N
        jScrollPane5.setViewportView(tfPair2CypherText);

        jScrollPane6.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane6.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
        jScrollPane6.setName("jScrollPane6"); // NOI18N

        tfPair2PlainText.setColumns(20);
        tfPair2PlainText.setRows(5);
        tfPair2PlainText.setName("tfPair2PlainText"); // NOI18N
        jScrollPane6.setViewportView(tfPair2PlainText);

        org.jdesktop.layout.GroupLayout jPanel6Layout = new org.jdesktop.layout.GroupLayout(jPanel6);
        jPanel6.setLayout(jPanel6Layout);
        jPanel6Layout.setHorizontalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel6Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel5)
                    .add(jLabel4))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 322, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jScrollPane5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 322, Short.MAX_VALUE))
                .addContainerGap(69, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        jPanel6Layout.setVerticalGroup(
            jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel6Layout.createSequentialGroup()
                .add(jPanel6Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel6Layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jLabel4)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 21, Short.MAX_VALUE)
                        .add(jLabel5))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jPanel6Layout.createSequentialGroup()
                        .add(4, 4, 4)
                        .add(jScrollPane6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 26, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(18, 18, 18)
                        .add(jScrollPane5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 25, Short.MAX_VALUE)))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout jPanel2Layout = new org.jdesktop.layout.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
            .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(jPanel8, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel2Layout.createSequentialGroup()
                .add(jPanel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 126, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(299, Short.MAX_VALUE))
            .add(jPanel2Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel2Layout.createSequentialGroup()
                    .addContainerGap(244, Short.MAX_VALUE)
                    .add(jPanel8, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(7, 7, 7)))
        );

        jTabbedPane1.addTab(resourceMap.getString("jPanel2.TabConstraints.tabTitle"), jPanel2); // NOI18N

        org.jdesktop.layout.GroupLayout mainPanelLayout = new org.jdesktop.layout.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 569, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jTabbedPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 592, Short.MAX_VALUE)
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        miOpenPlainTextFile.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_P, java.awt.event.InputEvent.META_MASK));
        miOpenPlainTextFile.setText(resourceMap.getString("miOpenPlainTextFile.text")); // NOI18N
        miOpenPlainTextFile.setName("miOpenPlainTextFile"); // NOI18N
        miOpenPlainTextFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                miOpenPlainTextFileActionPerformed(evt);
            }
        });
        fileMenu.add(miOpenPlainTextFile);

        miOpenCypherTextFile.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.META_MASK));
        miOpenCypherTextFile.setText(resourceMap.getString("miOpenCypherTextFile.text")); // NOI18N
        miOpenCypherTextFile.setName("miOpenCypherTextFile"); // NOI18N
        miOpenCypherTextFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                miOpenCypherTextFileActionPerformed(evt);
            }
        });
        fileMenu.add(miOpenCypherTextFile);

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(ddesbf.DDESBFApp.class).getContext().getActionMap(DDESBFView.class, this);
        miExit.setAction(actionMap.get("quit")); // NOI18N
        miExit.setName("miExit"); // NOI18N
        fileMenu.add(miExit);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        org.jdesktop.layout.GroupLayout statusPanelLayout = new org.jdesktop.layout.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(statusPanelSeparator, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 569, Short.MAX_VALUE)
            .add(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(statusMessageLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 373, Short.MAX_VALUE)
                .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(statusPanelLayout.createSequentialGroup()
                .add(statusPanelSeparator, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(statusPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(statusMessageLabel)
                    .add(statusAnimationLabel)
                    .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(3, 3, 3))
        );

        fcOpenFile.setName("fcOpenFile"); // NOI18N

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

private void miOpenPlainTextFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miOpenPlainTextFileActionPerformed
  int returnVal = fcOpenFile.showOpenDialog(this.mainPanel);
  if (returnVal == JFileChooser.APPROVE_OPTION) { //premuto tasto OK
    File plainTextFile=fcOpenFile.getSelectedFile();
    try {
      FileInputStream input=new FileInputStream(plainTextFile);
      int fileLength=new Long(plainTextFile.length()).intValue();
      byte[] vector=new byte[fileLength];
      input.read(vector);
      input.close();
      String strText=new String(vector);
      txtPlainText.setText(strText);
      statusMessageLabel.setText("Plain text file loaded");             
    }
           
    
    catch(IOException e){
        JOptionPane.showMessageDialog(this.mainPanel,"read error");   
    }
  }//GEN-LAST:event_miOpenPlainTextFileActionPerformed
}                                                   

private void miOpenCypherTextFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miOpenCypherTextFileActionPerformed
  
  int returnVal = fcOpenFile.showOpenDialog(this.mainPanel);
  if (returnVal == JFileChooser.APPROVE_OPTION) { //premuto tasto OK
      File cypherTextFile=fcOpenFile.getSelectedFile();
      try {
        FileInputStream input=new FileInputStream(cypherTextFile);
        int fileLength=new Long(cypherTextFile.length()).intValue();
        byte[] vector=new byte[fileLength];
        input.read(vector);
        input.close();
        String strText=new String(vector);
        txtCypherText.setText(strText);
        statusMessageLabel.setText("Cypher text file loaded");             
      
      
    }
    catch(IOException e){
        JOptionPane.showMessageDialog(this.mainPanel,"read error");   
    }
  }
}//GEN-LAST:event_miOpenCypherTextFileActionPerformed

private void btnCryptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCryptActionPerformed
  try{
    if((tfInputKey.getText().length()>15)&&(txtPlainText.getText().length()>7)){
      plainText=txtPlainText.getText().getBytes();
      FileOutputStream plainOutputStream=new FileOutputStream(new File("plainText.txt"));
      plainOutputStream.write(txtPlainText.getText().getBytes());
      plainOutputStream.close();
      
      byte[] key1=tfInputKey.getText().substring(0, 8).getBytes();
      byte[] key2=tfInputKey.getText().substring(8, 16).getBytes();
      


      
      DoubleDes dc=new DoubleDes(key1,key2);
      dc.encrypt("plainText.txt","cypherText.txt");
      
      File cypherTextFile= new File("cypherText.txt");
      FileInputStream input=new FileInputStream(cypherTextFile);
      int length=new Long(cypherTextFile.length()).intValue();
      cypherText=new byte[length];
      input.read(cypherText);
      input.close();
      String strText=new String(cypherText);
      txtCypherText.setText(strText);
  }
    else 
      JOptionPane.showMessageDialog(this.mainPanel,"Please insert plain text and the key.");
  }
  catch(IOException e){
    JOptionPane.showMessageDialog(this.mainPanel,"read error");   
  }
}//GEN-LAST:event_btnCryptActionPerformed

private void btnSetPair1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSetPair1ActionPerformed
  if((txtPlainText.getText().length()>7)&&(txtCypherText.getText().length()>7)){
    
    
  
    byte[] bytept1=plainText;
    byte[] bytect1=cypherText;
    
  
  
    
    String pt1="",ct1="";
    int helpint;
    for (int i=0;i<8;i++){
      helpint=0xFF & bytept1[i]; //conversione di un signed byte  in unsigned int 
      if (helpint<16)
        pt1+='0';
      pt1+=Integer.toHexString(helpint);

      helpint=0xFF & bytect1[i]; //conversione di un signed byte  in unsigned int 
      if (helpint<16)
        ct1+='0';
      ct1+=Integer.toHexString(helpint);


      
    }
  
    tfPair1PlainText.setText(pt1);
    tfPair1CypherText.setText(ct1);
    tfKeyOriginal.setText(DoubleDes.removeParity(tfInputKey.getText())+
                          DoubleDes.removeParity(tfInputKey.getText().substring(8)));
  }
  else
    JOptionPane.showMessageDialog(this.mainPanel,"Plain text or cypher text missing or incomplete");   
}//GEN-LAST:event_btnSetPair1ActionPerformed

private void btnDecryptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDecryptActionPerformed
  try{
    if((tfInputKey.getText().length()>15)&&(txtCypherText.getText().length()>7)){
      
        cypherText=txtCypherText.getText().getBytes();
      FileOutputStream plainOutputStream=new FileOutputStream(new File("cypherText.txt"));
      plainOutputStream.write(cypherText);
      plainOutputStream.close();
      
      byte[] key1=tfInputKey.getText().substring(0, 8).getBytes();
      byte[] key2=tfInputKey.getText().substring(8, 16).getBytes();
      
      
      DoubleDes dc=new DoubleDes(key1,key2);
      dc.decrypt("cypherText.txt","plainText.txt");
      
      File plainTextFile= new File("plainText.txt");
      FileInputStream input=new FileInputStream(plainTextFile);
      int length=new Long(plainTextFile.length()).intValue();
      plainText=new byte[length];
      input.read(plainText);
      input.close();
      String strText=new String(plainText);
      txtPlainText.setText(strText);
    }
    else 
      JOptionPane.showMessageDialog(this.mainPanel,"Please insert Cypher text and the key");  
  }
  catch(IOException e){
    JOptionPane.showMessageDialog(this.mainPanel,"read error");   
  }  
//GEN-LAST:event_btnDecryptActionPerformed
}

private void btnSetPair2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSetPair2ActionPerformed
if((txtPlainText.getText().length()>7)&&(txtCypherText.getText().length()>7)){
  
  byte[] bytept2=plainText;
  byte[] bytect2=cypherText;
  
  String pt2="",ct2="";
  int helpint;
  for (int i=0;i<8;i++){
    helpint=0xFF & bytept2[i]; //conversione di un signed byte  in unsigned int 
    if (helpint<16)
      pt2+='0';
    pt2+=Integer.toHexString(helpint);
    
    helpint=0xFF & bytect2[i]; //conversione di un signed byte  in unsigned int 
    if (helpint<16)
      ct2+='0';
    ct2+=Integer.toHexString(helpint);
  }
  tfPair2PlainText.setText(pt2);
  tfPair2CypherText.setText(ct2);
  tfKeyOriginal.setText(DoubleDes.removeParity(tfInputKey.getText())+
                          DoubleDes.removeParity(tfInputKey.getText().substring(8)));
  }
  else
    JOptionPane.showMessageDialog(this.mainPanel,"Plain text or cypher text missing or incomplete");   
}//GEN-LAST:event_btnSetPair2ActionPerformed

private void btnTimeTestActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnTimeTestActionPerformed
  String tableSize=bgTableSize.getSelection().getActionCommand();
  String alphabetType=new Long(cbAlphabetType.getSelectedIndex()).toString();
  String path=tfPath.getText();
  String args[]={path,"-t","-a",alphabetType,"-m",tableSize};
  try  
        {  
            Process process = Runtime.getRuntime().exec(args);  
            process.waitFor();  
            BufferedReader inputStreamReader  = new BufferedReader(new InputStreamReader(process.getInputStream()));  
            
            StringBuffer output = new StringBuffer();  
             
            for(String line;(line=inputStreamReader.readLine())!=null;)  
            {  
                output.append(line);  
            }  
            
  
            tfEstimatedTime.setText(output.toString());
        }  
        catch (InterruptedException e)  
        {  
            e.printStackTrace();  
        }  
        catch (IOException e)  
        {  
            JOptionPane.showMessageDialog(this.mainPanel,"./bin/mitma not found"); 
        }  
      
}//GEN-LAST:event_btnTimeTestActionPerformed

private void btnAttackActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAttackActionPerformed
                                            
  String tableSize=bgTableSize.getSelection().getActionCommand();
  String alphabetType=new Long(cbAlphabetType.getSelectedIndex()).toString();
  
  if((tfPair1PlainText.getText().length()<16)||
     (tfPair1CypherText.getText().length()<16)||
     (tfPair2PlainText.getText().length()<16)||
     (tfPair2CypherText.getText().length()<16)) {
    JOptionPane.showMessageDialog(this.mainPanel,"Insert Plain texts and Cypher texts first");
    return;
  }
  
  String pt1=tfPair1PlainText.getText();
  String ct1=tfPair1CypherText.getText();
  String pt2=tfPair2PlainText.getText();
  String ct2=tfPair2CypherText.getText();

  
  
  String path=tfPath.getText();
  String args[]={path,pt1,ct1,pt2,ct2,"-a",alphabetType,"-m",tableSize};
  try  
        {  
            Process process = Runtime.getRuntime().exec(args);  
            process.waitFor();  
            BufferedReader inputStreamReader  = new BufferedReader(new InputStreamReader(process.getInputStream()));  
            
            StringBuffer output = new StringBuffer();  
             
            for(String line;(line=inputStreamReader.readLine())!=null;)  
            {  
                output.append(line);  
            }  
            
  
            tfKeyFound.setText(DoubleDes.removeParity(output.toString())+
                               DoubleDes.removeParity(output.substring(8)));
        }  
        catch (InterruptedException e)  
        {  
             e.printStackTrace(); 
        }  
        catch (IOException e)  
        {  
          
          JOptionPane.showMessageDialog(this.mainPanel,"./bin/mitma not found");   
             
        }  
  
  

}//GEN-LAST:event_btnAttackActionPerformed

private void btnPathActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnPathActionPerformed
  int returnVal = fcOpenFile.showOpenDialog(this.mainPanel);
  if (returnVal == JFileChooser.APPROVE_OPTION) { //premuto tasto OK
    tfPath.setText(fcOpenFile.getSelectedFile().getAbsolutePath());
  }  
}//GEN-LAST:event_btnPathActionPerformed
  


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgCouple;
    private javax.swing.ButtonGroup bgTableSize;
    private javax.swing.JButton btnAttack;
    private javax.swing.JButton btnCrypt;
    private javax.swing.JButton btnDecrypt;
    private javax.swing.JButton btnPath;
    private javax.swing.JButton btnSetPair1;
    private javax.swing.JButton btnSetPair2;
    private javax.swing.JButton btnTimeTest;
    private javax.swing.JComboBox cbAlphabetType;
    private javax.swing.JFileChooser fcOpenFile;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JRadioButton jRadioButton3;
    private javax.swing.JRadioButton jRadioButton4;
    private javax.swing.JRadioButton jRadioButton5;
    private javax.swing.JRadioButton jRadioButton6;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JScrollPane jScrollPane5;
    private javax.swing.JScrollPane jScrollPane6;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem miOpenCypherTextFile;
    private javax.swing.JMenuItem miOpenPlainTextFile;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JTextField tfEstimatedTime;
    private javax.swing.JTextField tfInputKey;
    private javax.swing.JTextField tfKeyFound;
    private javax.swing.JTextField tfKeyOriginal;
    private javax.swing.JTextArea tfPair1CypherText;
    private javax.swing.JTextArea tfPair1PlainText;
    private javax.swing.JTextArea tfPair2CypherText;
    private javax.swing.JTextArea tfPair2PlainText;
    private javax.swing.JTextField tfPath;
    private javax.swing.JTextArea txtCypherText;
    private javax.swing.JTextArea txtPlainText;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private JDialog aboutBox;
    
    private byte[] cypherText=null;
    private byte[] plainText=null;
    
    
}
