/*----------------------------------------------------------------------------*/
/* Sviluppatore: Damiano Tognon 819675                                        */
/* Versione: 1.0                                                              */
/* Ultima modifica: 23/01/2009                                                */
/*----------------------------------------------------------------------------*/

/*---------------------------- DESCRIZIONE -----------------------------------*/
/*                                                                            */
/* Il modulo contiene il corpo delle funzioni principali utilizzate per       */
/* realizzare l'attacco "meet in the middle al Double DES.                    */
/* Per una descrizione dei parametri di ciascuna funzione, consultare l'header*/
/* "meet_in_the_middle.h".                                                    */
/*                                                                            */
/*----------------------------------------------------------------------------*/

/*---------------------------- HEADER FILES ----------------------------------*/

#include "meet_in_the_middle.h"

/*--------------------------- VARIABILI GLOBALI ------------------------------*/

uint matrixItemNumber[]={33554432,67108864,134217728,268435456};

uint addressBitSize[]={25,26,27,28};

uchar alphabets[6][33]={
  //valori numerici 0-9
	{0x30,0x32,0x34,0x36,0x38},
	//caratteri minuscoli a-z
	{0x60,0x62,0x64,0x66,0x68,0x6A,0x6C,0x6E,0x70,0x72,0x74,0x76,0x78,0x7A},
	//caratteri maiuscoli A-Z
	{0x40,0x42,0x44,0x46,0x48,0x4A,0x4C,0x4E,0x50,0x52,0x54,0x56,0x58,0x5A},
	//valori numerici 0-9 e caratteri minuscoli a-z
	{0x30,0x32,0x34,0x36,0x38,0x60,0x62,0x64,0x66,0x68,0x6A,0x6C,0x6E,0x70,0x72,0x74,0x76,0x78,0x7A},
	//valori numerici 0-9 e caratteri maiuscoli A-Z
	{0x30,0x32,0x34,0x36,0x38,0x40,0x42,0x44,0x46,0x48,0x4A,0x4C,0x4E,0x50,0x52,0x54,0x56,0x58,0x5A},
	//valori numerici 0-9, caratteri maiuscoli A-Z e caratteri minuscoli a-z
	{0x30,0x32,0x34,0x36,0x38,0x40,0x42,0x44,0x46,0x48,0x4A,0x4C,0x4E,0x50,0x52,0x54,0x56,0x58,0x5A,0x60,0x62,0x64,0x66,0x68,0x6A,0x6C,0x6E,0x70,0x72,0x74,0x76,0x78,0x7A},
};

uint alphabetsSize[]={5,14,14,19,19,33}; 

ullong alphaNKeys[6]={390625ULL,1475789056ULL,1475789056ULL,16983563041ULL,16983563041ULL,1406408618241ULL};

/*---------------------------- CREATETABLE -----------------------------------*/

uint createTable(uchar* PTB, uint nTable, uint memSize, uint keyType, table* currentTable){
	uint tableIndex,nItems=matrixItemNumber[memSize];
  uint nFirstKey=nTable*nItems;  //indice della prima chiave da creare
	uchar bufferKey[8],bufferCT[8];
	uchar* alphabet=alphabets[keyType];
	uint alphabetSize=alphabetsSize[keyType];
  uint N,i,j; //indici di varia utilità
	matrixItem* newItem;
	table matrix=(table)calloc(nItems,sizeof(matrixItem*)); //alloco la memoria
	*currentTable=matrix;
  if(matrix==NULL) //Non c'è memoria sufficiente a contenere la tabella
	  return NOT_ENOUGH_MEMORY;
	
	for (i=nFirstKey;i<nFirstKey+nItems;i++) {//riempio la prima tabella
		N=i;
		for(j=0;j<8;j++){ //Genero la i-esima chiave 
			bufferKey[7-j]=alphabet[N%alphabetSize];
			N/=alphabetSize;
		}
		des_encrypt(PTB,bufferCT,bufferKey);	//Genero la i-esima cifratura
		tableIndex=tableAddress(bufferCT,memSize);
		newItem=(matrixItem*)malloc(sizeof(matrixItem));
		newItem->TTB[0]=bufferCT[3];
		newItem->TTB[1]=bufferCT[4];
		newItem->TTB[2]=bufferCT[5];
		newItem->TTB[3]=bufferCT[6];
		newItem->TTB[4]=bufferCT[7];
		newItem->CKey[6]=(bufferKey[7]>>1)+(bufferKey[6]<<6);
		newItem->CKey[5]=(bufferKey[6]>>2)+(bufferKey[5]<<5);
		newItem->CKey[4]=(bufferKey[5]>>3)+(bufferKey[4]<<4);
		newItem->CKey[3]=(bufferKey[4]>>4)+(bufferKey[3]<<3);
		newItem->CKey[2]=(bufferKey[3]>>5)+(bufferKey[2]<<2);
		newItem->CKey[1]=(bufferKey[2]>>6)+(bufferKey[1]<<1);
		newItem->CKey[0]=(bufferKey[1]>>7)+(bufferKey[0]);
		newItem->next=matrix[tableIndex];
		matrix[tableIndex]=newItem;
	}
	return OK;
}

/*---------------------------- DESTROYTABLE ----------------------------------*/

void destroyTable(table currentTable, uint memSize){

  ullong i,size=matrixItemNumber[memSize];
	matrixItem* currentItem, *next;
  for (i=0;i<size;i++) {
	  currentItem=currentTable[i];
		while(currentItem!=NULL) {
			next=currentItem->next;
			free(currentItem);
			currentItem=next;
		}
	}
	free(currentTable);
}
  
/*-------------------------- MEET_IN_THE_MIDDLE ------------------------------*/

uchar* meet_in_the_middle(table currentTable,uint memSize,uint keyType,uchar* PT1, uchar* CT1, uchar* PT2, uchar* CT2){
  //genero la i-esima chiave di decifratura di CT1
	//decifro CT1 con la chiave al passo prec
	//utilizzo i primi x bit per indirizzare la tabella
	//se nella lista trovo una TTB uguale controllo che C_CKey(PT2)==D_iKey(CT2)
	uint i,j,N,tableIndex;
	uchar bufferKey1[8],bufferKey2[8],bufferPT[8],CPT2[8],DCT2[8];
	uchar* keyFound;
	uchar* alphabet=alphabets[keyType];
	uint alphabetSize=alphabetsSize[keyType];
	uint nKeys=alphaNKeys[keyType];
	
	for(i=0;i<nKeys;i++) { //devo provare tutte le chiavi!!
	  N=i;
		for(j=0;j<8;j++){ //Genero la i-esima chiave 
			bufferKey2[7-j]=alphabet[N%alphabetSize];
			N/=alphabetSize;
		}
		des_decrypt(CT1,bufferPT,bufferKey2);
	  tableIndex=tableAddress(bufferPT,memSize);
		matrixItem* currentItem=currentTable[tableIndex];
		
		while(currentItem!=NULL) { //controllo tutti gli elementi in lista
		  if (currentItem->TTB[0]==bufferPT[3]&&currentItem->TTB[1]==bufferPT[4]&&
					currentItem->TTB[2]==bufferPT[5]&&currentItem->TTB[3]==bufferPT[6]&&
					currentItem->TTB[4]==bufferPT[7]) { //Cif. decif. coincidono
					//espando currentItem.Ckey su 8 byte e memorizzo in bufferKey
				addParity(currentItem->CKey,bufferKey1);
				des_encrypt(PT2,CPT2,bufferKey1);
				des_decrypt(CT2,DCT2,bufferKey2);
				if(CPT2[0]==DCT2[0]&&CPT2[1]==DCT2[1]&&CPT2[2]==DCT2[2]&&CPT2[3]==DCT2[3]&&  
					 CPT2[4]==DCT2[4]&&CPT2[5]==DCT2[5]&&CPT2[6]==DCT2[6]&&CPT2[7]==DCT2[7]){
				  //restituisco i 56+56 bit delle due chiavi trovate
					keyFound=malloc(14*sizeof(uchar));
					memcpy(keyFound,currentItem->CKey,7);
					removeParity(bufferKey2,keyFound+7);
					return keyFound;	 
				}
			}
			currentItem=currentItem->next;
		}
	}
  return NULL;
}

/*---------------------------- DES_ENCRYPT -----------------------------------*/


	/* key, plaintext, ciphertext must be 8 byte blocks */ 
void des_encrypt(uchar* PTB, uchar* CTB, uchar* key) {
	des_key_schedule sched;
	des_set_key_unchecked((DES_cblock*)key,sched);			 
	des_ecb_encrypt((DES_cblock*)PTB,(DES_cblock*)CTB,sched,DES_ENCRYPT);
}

/*---------------------------- DES_DECRYPT -----------------------------------*/


void des_decrypt(uchar* CTB, uchar* PTB, uchar* key) {
	des_key_schedule sched;
  des_set_key_unchecked((DES_cblock*)key,sched);			 
	des_ecb_encrypt((DES_cblock*)CTB,(DES_cblock*)PTB,sched,DES_DECRYPT);
}

/*---------------------------- TABLEADDRESS ----------------------------------*/


uint tableAddress(uchar bufferCT[],uint memSize) {
  uint tableIndex;
	switch (memSize) {
	  case MB512: //primi 25 bit dell'encryption
		  tableIndex=(bufferCT[3]>>7)+(bufferCT[2]<<1)+
								 (bufferCT[1]<<9)+(bufferCT[0]<<17); 
			break;
		case GB1:	//primi 26 bit dell'encryption
		  tableIndex=(bufferCT[3]>>6)+(bufferCT[2]<<2)+
								 (bufferCT[1]<<10)+(bufferCT[0]<<18); 
			break;
		case GB2:	//primi 27 bit dell'encryption
			tableIndex=(bufferCT[3]>>5)+(bufferCT[2]<<3)+
								 (bufferCT[1]<<11)+(bufferCT[0]<<19); 
			break;
		case GB4:	//primi 28 bit dell'encryption
			tableIndex=(bufferCT[3]>>4)+(bufferCT[2]<<4)+
								 (bufferCT[1]<<12)+(bufferCT[0]<<20); 
			break;
	}
	return tableIndex;
}

/*---------------------------- ADDPARITY -------------------------------------*/


void addParity(uchar source[], uchar dest[]) {
	dest[7]=source[6]<<1;
	dest[6]=(source[5]<<2) + (source[6]>>6);
	dest[5]=(source[4]<<3) + (source[5]>>5);
	dest[4]=(source[3]<<4) + (source[4]>>4);
	dest[3]=(source[2]<<5) + (source[3]>>3);
	dest[2]=(source[1]<<6) + (source[2]>>2);
	dest[1]=(source[0]<<7) + (source[1]>>1);
	dest[0]=source[0];	
}


/*---------------------------- REMOVEPARITY ----------------------------------*/

void removeParity(uchar source[], uchar dest[]) {
	dest[6]=(source[7]>>1)+((source[6]>>1)<<7);
	dest[5]=(source[6]>>2)+((source[5]>>1)<<6);
	dest[4]=(source[5]>>3)+((source[4]>>1)<<5);
	dest[3]=(source[4]>>4)+((source[3]>>1)<<4);
	dest[2]=(source[3]>>5)+((source[2]>>1)<<3);
	dest[1]=(source[2]>>6)+((source[1]>>1)<<2);
	dest[0]=(source[1]>>7)+((source[0]>>1)<<1);
} 

/*------------------------------ HEXTOCHAR -----------------------------------*/

int hexToChar(const char in[], uchar out[]) {
  int i;
  if (strlen(in)<16) return INPUT_ERROR;

	for(i=7;i>=0;i--){
	
	  if ((in[i*2+1]>='0')&&(in[i*2+1]<='9')) // carattere '0' - '9'
	    out[i]=in[i*2+1]-'0';
		else if ((in[i*2+1]>='A')&&(in[i*2+1]<='F')) //carattere 'A' - 'F'
		  out[i]=in[i*2+1]-0x37;
		else if ((in[i*2+1]>='a')&&(in[i*2+1]<='f')) //carattere 'a' - 'f'
		  out[i]=in[i*2+1]-0x57;
		else
		  return INPUT_ERROR;
		
		if ((in[i*2]>='0')&&(in[i*2]<='9')) // carattere '0' - '9'
      out[i]+=(in[i*2]-'0')<<4;
		else if ((in[i*2]>='A')&&(in[i*2]<='F')) //carattere 'A' - 'F'
		  out[i]+=(in[i*2]-0x37)<<4;
		else if ((in[i*2]>='a')&&(in[i*2]<='f')) //carattere 'a' - 'f'
		  out[i]+=(in[i*2]-0x57)<<4;
		else
		  return INPUT_ERROR;			 	
	}
	return OK;
}


