/*----------------------------------------------------------------------------*/
/* Sviluppatore: Damiano Tognon 819675                                        */
/* Versione: 1.0                                                              */
/* Ultima modifica: 23/01/2009                                                */
/*----------------------------------------------------------------------------*/

/*----------------------------DESCRIZIONE-------------------------------------*/
/*                                                                            */
/* Contiene i protopi delle funzioni realizzate con la descrizione dei        */
/* parametri per un corretto utilizzo.                                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/  

/*-------------------- DIRETTIVE PER IL PREPROCESSORE ------------------------*/

#ifndef _meet_in_the_middle_h
#define _meet_in_the_middle_h

/*---------------------------- HEADER FILES ----------------------------------*/

#include "data_str.h"
#include <string.h> //per la memcpy
#include <openssl/des.h>

/*------------------------- createTable --------------------------------------*/
/* Descrizione:                                                               */                                                                            
/*   La funzione crea in memoria un hash table di "matrixItem", ciascuno dei  */
/*   quali contiene la cifratura del primo testo in chiaro con la chiave usata*/
/*   per generarla. Gli elementi sono ordinati secondo i primi x bit delle    */
/*   cifrature, con x variabile a seconda della dimensione specificata        */
/*   per la tabella.                                                          */ 
/*                                                                            */
/* Parametri:                                                                 */
/*   PTB: Testo in chiaro (8 byte)                                            */
/*   nTable: numero della tabella (necessario per generare le chiavi)         */
/*   memSize: Dimensione della tabella (vedi data_str.h)                      */
/*   keyType: Tipo della chiave cercata (numerica,..) (vedi data_str.h)       */
/*   currentTable: puntatore alla tabella che verrà creata                    */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   OK - tabella creata correttamente                                        */
/*   NOT_ENOUGH_MEMORY - memoria insufficiente a contenere la tabella         */
/*                                                                            */
/*----------------------------------------------------------------------------*/

uint createTable(uchar* PTB, uint nTable, uint memSize, uint keyType, table* currentTable);

/*------------------------- destroyTable -------------------------------------*/  
/* Descrizione:                                                               */                                                                            
/*   Rimuove dalle memoria la tabella creata.                                 */ 
/*                                                                            */
/* Parametri:                                                                 */
/*   currentTable: La tabella da eliminare                                    */
/*   memSize: La dimensione della tabella da eliminare                        */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   Nessuno                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

void destroyTable(table currentTable, uint memSize);

/*---------------------- meet_in_the_middle ----------------------------------*/  
/* Descrizione:                                                               */                                                                           
/*   La funzione realizza l'attacco "meet in the middle", cercando nella      */
/*   tabella specificata la chiave che soddisfa le due coppie testo in chiaro */
/*   testo cifrato.                                                           */
/*                                                                            */
/* Parametri:                                                                 */
/*   currentTable: La tabella in cui effettuare la ricerca                    */
/*   memSize: La dimensione della tabella                                     */
/*   keyType: Il tipol della chiave cercata (vedi data_str.h)                 */
/*   PT1: Il primo testo in chiaro                                            */
/*   CT1: Il primo testo cifrato                                              */
/*   PT2: Il secondo testo in chiaro                                          */
/*   CT2: Il secondo testo cifrato                                            */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   KeyFound - la chiave trovata in caso di esito positivo                   */
/*   NULL - in caso di esito negativo della ricerca                           */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

uchar* meet_in_the_middle(table currentTable,uint memSize,uint keyType,uchar* PT1, uchar* CT1, uchar* PT2, uchar* CT2);

/*------------------------- des_encrypt --------------------------------------*/  
/* Descrizione:                                                               */                                                                          
/*   Cifra un testo in chiaro in testo cifrato tramite DES e la chiave        */
/*   specificata.                                                             */ 
/*                                                                            */
/* Parametri:                                                                 */
/*   PTB: Il testo in chiaro da cifrare  (8 Byte)                             */
/*   CTB: Il testo cifrato ottenuto      (8 Byte)                             */
/*   key: La chiave utilizzata per la cifratura (8 Byte)                      */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   Nessuno                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

void des_encrypt(uchar* PTB, uchar* CTB, uchar* key);

/*------------------------- des_decrypt --------------------------------------*/  
/* Descrizione:                                                               */                                                                            
/*   Decifra un testo cifrato in testo in chiaro tramite DES e la chiave      */
/*   specificata.                                                             */ 
/*                                                                            */
/* Parametri:                                                                 */
/*   CTB: Il testo cifrato da decifrare  (8 Byte)                             */
/*   PTB: Il testo in chiaro ottenuto    (8 Byte)                             */
/*   key: La chiave utilizzata per la decifratura (8 Byte)                    */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   Nessuno                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/

void des_decrypt(uchar* CTB, uchar* PTB, uchar* key);

/*------------------------- tableAddress -------------------------------------*/  
/* Descrizione:                                                               */                                                                           
/*   Dato un testo cifrato e la dimensione della tabella in memoria, la       */
/*   funzione restituisce i primi x bit della cifratura da utilizzare come    */
/*   indice per indirizzare l'hash table.                                     */    
/*                                                                            */
/* Parametri:                                                                 */
/*   CTB: cifratura (8 Byte)                                                  */
/*   memSize: dimensione della tabella in memoria (vedi data_str.h)           */    
/*                                                                            */
/* Valori di ritorno:                                                         */
/*    L'indice richiesto                                                      */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

uint tableAddress(uchar bufferCT[],uint memSize);

/*--------------------------- addParity --------------------------------------*/  
/* Descrizione:                                                               */                                                                          
/*   La funzione prende in input una chiave di 56 bit e ritorna la chiave con */
/*   l'aggiunta dei bit di parità.                                            */
/*                                                                            */
/* Parametri:                                                                 */
/*   source: La chiave in input (7 Byte)                                      */
/*   dest:   La chiave in output (8 Byte)                                     */ 
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   Nessuno                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/

void addParity(uchar source[], uchar dest[]);

/*------------------------- removeParity -------------------------------------*/  
/* Descrizione:                                                               */                                                                           
/*   La funzione rimuove dalla chiave in input gli 8 bit di parità.           */
/*                                                                            */
/* Parametri:                                                                 */
/*   source: La chiave in input (8 Byte)                                      */
/*   dest:   La chiave in output (7 Byte)                                     */ 
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   Nessuno                                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

void removeParity(uchar source[], uchar dest[]);

/*------------------------- hexToChar ----------------------------------------*/  
/* Descrizione:                                                               */                                                                           
/*   La funzione serve a convertire la stringa esadecimale in input           */
/*   in un vettore d 8 byte.                                                  */ 
/*                                                                            */
/* Parametri:                                                                 */
/*   in: La stringa esadecimale in input (16 caratteri)                       */
/*   out: Il vettore di byte di output (8 Byte)                               */
/*                                                                            */
/* Valori di ritorno:                                                         */
/*   OK - La conversione è andata a buon fine                                 */
/*   INPUT_ERROR: La stringa in input sono presenti caratteri non previsti    */
/*                                                                            */
/*----------------------------------------------------------------------------*/ 

int hexToChar(const char in[], uchar out[]);

#endif
