/*----------------------------------------------------------------------------*/
/* Sviluppatore: Damiano Tognon 819675                                        */
/* Versione: 1.0                                                              */
/* Ultima modifica: 23/01/2009                                                */
/*----------------------------------------------------------------------------*/

/*----------------------------DESCRIZIONE-------------------------------------*/
/*                                                                            */
/* Il modulo contiene il main dell'applicazione. La sintassi per lanciare il  */
/* programma da riga di comando è la seguente:                                */
/*                                                                            */
/* mitma (-t | (<pt1> <ct1> <pt2> <ct2>)) [-a <alpha_type>] [-m <mem_size>]   */
/*                                                                            */
/* pt1: Plain Text Block 1, il primo testo in chiaro                          */
/* ct1: Cypher Text Block 1, la cifratura relativa a ct1                      */    
/* pt2: Plain Text Block 2, il secondo testo in chiaro                        */
/* ct2: Cypher Text Block 2, la cifratura relativa a ct2                      */
/* alpha_type: L'alfabeto scelto, i possibili valori sono:                    */
/*                                                                            */
/*   0 - numeric character																										*/
/*   1 - lower case character {default}                                       */
/*   2 - upper case character                                                 */
/*   3 - numeric and lower case character                                     */
/*   4 - numeric and upper case character                                     */
/*   5 - numeric, lower case and upper case charater                          */
/*                                                                            */
/* mem_size: La dimensione della tabella in memoria, i possibili valori sono: */
/*                                                                            */
/*   0 - 512 MB                                                               */
/*   1 - 1 GB {default}                                                       */
/*   2 - 2 GB                                                                 */
/*   3 - 4 GB                                                                 */
/*----------------------------------------------------------------------------*/

/*---------------------------- HEADER FILES ----------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "data_str.h"
#include "meet_in_the_middle.h"

/*-------------------------- VARIABILI GLOBALI -------------------------------*/

extern uint matrixItemNumber[];
extern ullong alphaNKeys[];

/*-------------------------------- MAIN --------------------------------------*/

int main (int argc, const char * argv[]) {

  //------- Codice per formattare correttamente i valori in input ------------
	//---------- Se l'input è mal formattato return INPUT_ERROR ----------------   
	
	uint alphaType=NUM;
	uint memSize=MB512;
	uint timeTest=FALSE;
 	uchar PT1[8]={'s','e','c','u','r','i','t','y'};
	uchar CT1[8]={0x10,0x8E,0x46,0x76,0xDA,0xF6,0x7D,0x6B};
	uchar PT2[8]={'i','n','e','n','o','u','g','h'};
	uchar CT2[8]={0x24,0x28,0x09,0xC6,0x76,0x69,0x89,0x61};
	uint state=1, inputIndex;
	
	
	for(inputIndex=1;inputIndex<=argc;inputIndex++) {
	  switch (state) {
			case 1:
			  if(strcmp("-t",argv[inputIndex])==0) { //timetest
				  timeTest=TRUE;
					state=2;
				}
				else {  //lettura di pt1
				  if (hexToChar(argv[inputIndex],PT1) == INPUT_ERROR) 
					return INPUT_ERROR;
					state=6;
				}
				break;
				
			case 2:
			  if (inputIndex==argc) state=OK;
				else if (strcmp("-a",argv[inputIndex])==0) state=3;
				else if (strcmp("-m",argv[inputIndex])==0) state=5;
				else return INPUT_ERROR;
				break;
				
			case 3:
			  alphaType=strtol(argv[inputIndex],NULL,10);
				if ((alphaType<NUM)||(alphaType>ALL))
			    return INPUT_ERROR;
				state=4;	
				break;
				
			case 4:
			  if (inputIndex==argc) state=OK;
				else if (strcmp("-m",argv[inputIndex])==0) state=5;
				else return INPUT_ERROR;
				break;
				
			case 5:
			  state=OK;
			  memSize=strtol(argv[inputIndex],NULL,10);
				if((memSize<MB512)||(memSize>GB4))
				  return INPUT_ERROR;
				break;
				
			case 6: //lettura di ct1
			  if (hexToChar(argv[inputIndex],CT1) == INPUT_ERROR) 
					return INPUT_ERROR;
				state=7;
				break;
				
			case 7: //lettura di pt2
			  if (hexToChar(argv[inputIndex],PT2) == INPUT_ERROR) 
					return INPUT_ERROR;
				state=8;
				break;
				
			case 8: //lettura di ct2
			  if (hexToChar(argv[inputIndex],CT2) == INPUT_ERROR) 
					return INPUT_ERROR;
				state=2;
				break;
			  
					
			case OK:
			  break;
	  }
	}
	if(state!=OK) return INPUT_ERROR;
	

  //------ VARIABILI LOCALI -------------
	
  ullong totalEntryNum=alphaNKeys[alphaType]; //numero totale di elementi
	table currentTable;
	uchar* keyFound;
	char key1[8], key2[8];
	uint i;
	double result;
	time_t start,end;
	uint timeEstimated;
	uint matrixNum=totalEntryNum/matrixItemNumber[memSize];
	if(totalEntryNum%matrixItemNumber[memSize]!=0) matrixNum++; 
	
	//-------- opzione TIMETEST ----------- 

	if(timeTest==TRUE) {
		
		time(&start);
		result=createTable(PT1,0,memSize,alphaType,&currentTable);
		if(result == NOT_ENOUGH_MEMORY)
				return result;
		keyFound=meet_in_the_middle(currentTable,memSize,alphaType,PT1,CT1,PT2,CT2);
		destroyTable(currentTable,memSize); 
		time(&end);
		if (keyFound != NULL)
			timeEstimated=difftime(end,start);
		else		
		  timeEstimated=difftime(end,start)*matrixNum;
		printf("%d days %d hours %d mins %d secs",timeEstimated/86400,(timeEstimated/3600)%24,(timeEstimated/60)%60,timeEstimated%60);	
		return timeEstimated;
	}
	
	//-------- RICERCA DELLA CHIAVE -----------
	
	else {
	  for (i=0; i<matrixNum;i++) {
			result=createTable(PT1,i,memSize,alphaType,&currentTable);
		  if(result == NOT_ENOUGH_MEMORY)
				return result;
			keyFound=meet_in_the_middle(currentTable,memSize,alphaType,PT1,CT1,PT2,CT2);
			if (keyFound != NULL) { 
			  addParity(keyFound,(uchar*)key1);
				addParity(keyFound+7,(uchar*)key2);
			  for(i=0;i<8;i++)
				  printf("%c",key1[i]);
				for(i=0;i<8;i++)
				  printf("%c",key2[i]);
			  return OK;
			}
			destroyTable(currentTable,memSize); 
		}
		printf("Key not found :-(");
		return KEY_NOT_FOUND;
	}
}




 
